@@ -1,6 +1,8 @@
 """Collections of library function names.
 """
 
+import rbc.rbclib
+
 
 class Library:
     """Base class for a collection of library function names.
@@ -24,6 +26,8 @@ class Library:
             r = LLVMIntrinsics()
         elif libname == 'omniscidb':
             r = OmnisciDB()
+        elif libname == 'rbclib':
+            r = RBCLib()
         else:
             raise ValueError(f'Unknown library {libname}')
         _cache[libname] = r
@@ -52,6 +56,8 @@ class OmnisciDB(Library):
 
     _function_names = list('''
     allocate_varlen_buffer set_output_row_size
+    TableFunctionManager_error_message TableFunctionManager_set_output_row_size
+    table_function_error
     '''.strip().split())
 
 
@@ -305,3 +311,14 @@ class Libdevice(Library):
     ull2double_ru ull2double_rz ull2float_rd ull2float_rn ull2float_ru
     ull2float_rz ullmax ullmin umax umin umul24 umul64hi umulhi urhadd
     usad y0 y0f y1 y1f yn ynf '''.strip().split())
+
+
+class RBCLib(Library):
+
+    name = 'rbclib'
+    # _function_names contains the list of functions which is exported by the
+    # library. See Library.check()
+    from rbc.rbclib import FUNCTION_NAMES as _function_names
+
+    def __init__(self):
+        rbc.rbclib.load_inside_llvm()
