@@ -34,6 +34,26 @@ if tblib is not None:
     tblib.pickling_support.install()
 
 
+class TServerSocket(thr.transport.TServerSocket):
+
+    def close(self):
+        # Copied from https://github.com/Thriftpy/thriftpy2/pull/184
+        # Remove when using pythrift2 > 0.4.14
+        if not self.sock:
+            return
+
+        try:
+            self.sock.shutdown(socket.SHUT_RDWR)
+        except OSError:
+            pass
+
+        try:
+            self.sock.close()
+        except OSError:
+            pass
+        self.sock = None
+
+
 class Processor(thr.thrift.TProcessor):
 
     def __init__(self, server, service, handler):
@@ -122,15 +142,16 @@ class Server(object):
                         args=(dispatcher, thrift_file, options))
         p.start()
         start = time.time()
+        number_of_tries = 0
         while time.time() < start + startup_time:
+            number_of_tries += 1
             try:
                 socket.create_connection(
-                    (options['host'], options['port']), timeout=0.1)
+                    (options['host'], options['port']), timeout=0.2)
             except ConnectionRefusedError:
                 time.sleep(0.5)
             except Exception as msg:
-                print('Connection failed: `%s`, trying again in 0.5 secs..'
-                      % (msg))
+                warnings.warn(f'Connection failed: `{msg}`, trying again in 0.5 secs..')
                 time.sleep(0.5)
             else:
                 break
@@ -141,8 +162,12 @@ class Server(object):
                 p.terminate()
             raise RuntimeError(
                 'failed to start up rpc_thrift server'
-                ' (was alive={}, startup_time={}s)'
-                .format(is_alive, startup_time))
+                f' (was alive={is_alive}, startup time={startup_time}s,'
+                f' elapsed time={time.time() - start})')
+        if number_of_tries > 1:
+            warnings.warn(
+                f'More than one try ({number_of_tries}) in starting up rpc_thrift_server.'
+                f' Total elapsed time is {time.time() - start} seconds.')
         return p
 
     def _serve(self):
@@ -161,7 +186,7 @@ class Server(object):
             s_proc = Processor(self, service, self._dispatcher(self, debug=self.debug))
         server = thr.server.TThreadedServer(
             s_proc,
-            thr.transport.TServerSocket(**self.options),
+            TServerSocket(**self.options),
             iprot_factory=thr.protocol.TBinaryProtocolFactory(),
             itrans_factory=thr.transport.TBufferedTransportFactory())
         server.serve()
